@@ -35,6 +35,23 @@
 uint16_t abort_fails;	/* count of failures */
 bool	 abort_msg_emitted;
 
+/*
+ *  pr_yaml()
+ *	print to yaml file if it is open
+ */
+int pr_yaml(FILE *fp, const char *const fmt, ...)
+{
+	va_list ap;
+	int ret = 0;
+
+	va_start(ap, fmt);
+	if (fp)
+		ret = vfprintf(fp, fmt, ap);
+	va_end(ap);
+
+	return ret;
+}
+
 /*
  *  print()
  *	print some debug or info messages
@@ -51,20 +68,23 @@ int print(
 	if (opt_flags & flag) {
 		char buf[4096];
 		const char *type = "";
-		int n;
 
 		if (flag & PR_ERROR)
-			type = "error";
+			type = "error:";
 		if (flag & PR_DEBUG)
-			type = "debug";
+			type = "debug:";
 		if (flag & PR_INFO)
-			type = "info";
+			type = "info: ";
 		if (flag & PR_FAIL)
-			type = "fail";
+			type = "fail: ";
 
-		n = snprintf(buf, sizeof(buf), "%s: [%i] ", type, getpid());
-		ret = vsnprintf(buf + n, sizeof(buf) - n, fmt, ap);
-		fprintf(fp, "%s: %s", app_name, buf);
+		if (opt_flags & OPT_FLAGS_LOG_BRIEF) {
+			ret = vfprintf(fp, fmt, ap);
+		} else {
+			int n = snprintf(buf, sizeof(buf), "%s [%i] ", type, getpid());
+			ret = vsnprintf(buf + n, sizeof(buf) - n, fmt, ap);
+			fprintf(fp, "%s: %s", app_name, buf);
+		}
 		fflush(fp);
 
 		if (flag & PR_FAIL) {
