@@ -78,8 +78,12 @@ int stress_bigheap(
 			opt_bigheap_growth = MIN_BIGHEAP_GROWTH;
 	}
 again:
+	if (!opt_do_run)
+		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
+		if (errno == EAGAIN)
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -94,8 +98,8 @@ again:
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
-			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
-				name, WTERMSIG(status), instance);
+			pr_dbg(stderr, "%s: child died: %s (instance %d)\n",
+				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
