@@ -53,8 +53,12 @@ int stress_brk(
 	const size_t page_size = stress_get_pagesize();
 
 again:
+	if (!opt_do_run)
+		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
+		if (errno == EAGAIN)
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -69,8 +73,8 @@ again:
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
-			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
-				name, WTERMSIG(status), instance);
+			pr_dbg(stderr, "%s: child died: %s (instance %d)\n",
+				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
@@ -81,7 +85,7 @@ again:
 			}
 		}
 	} else if (pid == 0) {
-		uint8_t *ptr, *start_ptr;
+		uint8_t *start_ptr;
 		bool touch = !(opt_flags & OPT_FLAGS_BRK_NOTOUCH);
 
 		/* Make sure this is killable by OOM killer */
@@ -95,7 +99,7 @@ again:
 		}
 
 		do {
-			ptr = sbrk((intptr_t)page_size);
+			uint8_t *ptr = sbrk((intptr_t)page_size);
 			if (ptr == (void *)-1) {
 				if (errno == ENOMEM) {
 					nomems++;
@@ -116,7 +120,6 @@ again:
 			}
 			(*counter)++;
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
-		free(ptr);
 	}
 	if (restarts + nomems > 0)
 		pr_dbg(stderr, "%s: OOM restarts: %" PRIu32 ", out of memory restarts: %" PRIu32 ".\n",
