@@ -154,7 +154,7 @@ int stress_chmod(
 		 *  other chmod stressors have already created it and changed the
 		 *  permission bits. If so, wait a while and retry (ugh).
 		 */
-		if ((fd = open(filename, O_CREAT | O_APPEND | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
+		if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0) {
 			if (errno == EPERM || errno == EACCES) {
 				(void)usleep(100000);
 				continue;
@@ -165,9 +165,11 @@ int stress_chmod(
 		break;
 	} while (opt_do_run && ++retries < 100);
 
-	if (retries > 0)
-		pr_dbg(stderr, "%s: chmod: file %s took %d retries to create (instance %" PRIu32 ")\n",
+	if (retries >= 100) {
+		pr_err(stderr, "%s: chmod: file %s took %d retries to create (instance %" PRIu32 ")\n",
 			name, filename, retries, instance);
+		goto tidy;
+	}
 
 	for (i = 0; modes[i]; i++)
 		all_mask |= modes[i];
