@@ -26,9 +26,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <math.h>
 #include <complex.h>
 #include <sys/time.h>
+#include <unistd.h>
 #include "stress-ng.h"
 
 #define GAMMA 	(0.57721566490153286060651209008240243104215933593992L)
@@ -68,10 +70,20 @@ typedef struct {
 	const stress_cpu_func	func;	/* the stressor function */
 } stress_cpu_stressor_info_t;
 
+/*
+ *  opt_cpu_load_slice:
+ *	< 0   - number of iterations per busy slice
+ *	= 0   - random duration between 0..0.5 seconds
+ *	> 0   - milliseconds per busy slice
+ */
+static int32_t opt_cpu_load_slice = -64;
 static int32_t opt_cpu_load = 100;
 static stress_cpu_stressor_info_t *opt_cpu_stressor;
 static stress_cpu_stressor_info_t cpu_methods[];
 
+/* Don't make this static to ensure dithering does not get optimised out */
+uint8_t pixels[STRESS_CPU_DITHER_X][STRESS_CPU_DITHER_Y];
+
 void stress_set_cpu_load(const char *optarg) {
 	opt_cpu_load = opt_long("cpu load", optarg);
 	if ((opt_cpu_load < 0) || (opt_cpu_load > 100)) {
@@ -80,16 +92,25 @@ void stress_set_cpu_load(const char *optarg) {
 	}
 }
 
+void stress_set_cpu_load_slice(const char *optarg)
+{
+	opt_cpu_load_slice = opt_long("cpu load slice", optarg);
+	if ((opt_cpu_load < -5000) || (opt_cpu_load > 5000)) {
+		fprintf(stderr, "CPU load must in the range -5000 to 5000.\n");
+		exit(EXIT_FAILURE);
+	}
+}
+
 /*
  *  stress_cpu_sqrt()
  *	stress CPU on square roots
  */
-static void stress_cpu_sqrt(const char *name)
+static void HOT stress_cpu_sqrt(const char *name)
 {
 	int i;
 
 	for (i = 0; i < 16384; i++) {
-		uint64_t rnd = mwc();
+		uint64_t rnd = mwc32();
 		double r = sqrt((double)rnd) * sqrt((double)rnd);
 		if ((opt_flags & OPT_FLAGS_VERIFY) &&
 		    (uint64_t)rint(r) != rnd) {
@@ -132,7 +153,7 @@ static void stress_cpu_loop(const char *name)
  *  stress_cpu_gcd()
  *	compute Greatest Common Divisor
  */
-static void stress_cpu_gcd(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_gcd(const char *name)
 {
 	uint32_t i, i_sum = 0;
 	const uint32_t sum = 63000868UL;
@@ -159,7 +180,7 @@ static void stress_cpu_gcd(const char *name)
  *	various bit manipulation hacks from bithacks
  *	https://graphics.stanford.edu/~seander/bithacks.html
  */
-static void stress_cpu_bitops(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_bitops(const char *name)
 {
 	uint32_t i, i_sum = 0;
 	const uint32_t sum = 0x8aadcaab;
@@ -212,7 +233,7 @@ static void stress_cpu_bitops(const char *name)
  *  stress_cpu_trig()
  *	simple sin, cos trig functions
  */
-static void stress_cpu_trig(const char *name)
+static void HOT stress_cpu_trig(const char *name)
 {
 	int i;
 	long double d_sum = 0.0;
@@ -248,7 +269,7 @@ static void stress_cpu_trig(const char *name)
  *  stress_cpu_hyperbolic()
  *	simple hyperbolic sinh, cosh functions
  */
-static void stress_cpu_hyperbolic(const char *name)
+static void HOT stress_cpu_hyperbolic(const char *name)
 {
 	int i;
 	double d_sum = 0.0;
@@ -260,21 +281,21 @@ static void stress_cpu_hyperbolic(const char *name)
 		{
 			d_sum += (coshl(theta) * sinhl(theta));
 			d_sum += (cosh(theta) * sinh(theta));
-			d_sum += (coshf(theta) * sinhf(theta));
+			d_sum += (double)(coshf(theta) * sinhf(theta));
 		}
 		{
 			long double theta2 = theta * 2.0;
 
 			d_sum += coshl(theta2);
 			d_sum += cosh(theta2);
-			d_sum += coshf(theta2);
+			d_sum += (double)coshf(theta2);
 		}
 		{
 			long double theta3 = theta * 3.0;
 
 			d_sum += sinhl(theta3);
 			d_sum += sinh(theta3);
-			d_sum += sinhf(theta3);
+			d_sum += (double)sinhf(theta3);
 		}
 	}
 	double_put(d_sum);
@@ -284,7 +305,7 @@ static void stress_cpu_hyperbolic(const char *name)
  *  stress_cpu_rand()
  *	generate lots of pseudo-random integers
  */
-static void stress_cpu_rand(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_rand(const char *name)
 {
 	int i;
 	uint32_t i_sum = 0;
@@ -292,17 +313,38 @@ static void stress_cpu_rand(const char *name)
 
 	MWC_SEED();
 	for (i = 0; i < 16384; i++)
-		i_sum += mwc();
+		i_sum += mwc32();
 
 	if ((opt_flags & OPT_FLAGS_VERIFY) && (i_sum != sum))
 		pr_fail(stderr, "%s: rand error detected, failed sum of pseudo-random values\n", name);
 }
 
+/*
+ *  stress_cpu_rand48()
+ *	generate random values using rand48 family of functions
+ */
+static void HOT OPTIMIZE3 stress_cpu_rand48(const char *name)
+{
+	int i;
+	double d = 0;
+	long int l = 0;
+
+	(void)name;
+
+	srand48(0x0defaced);
+	for (i = 0; i < 16384; i++) {
+		d += drand48();
+		l += lrand48();
+	}
+	double_put(d);
+	uint64_put(l);
+}
+
 /*
  *  stress_cpu_nsqrt()
  *	iterative Newton–Raphson square root
  */
-static void stress_cpu_nsqrt(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_nsqrt(const char *name)
 {
 	int i;
 	const long double precision = 1.0e-12;
@@ -337,7 +379,7 @@ static void stress_cpu_nsqrt(const char *name)
  *  stress_cpu_phi()
  *	compute the Golden Ratio
  */
-static void stress_cpu_phi(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_phi(const char *name)
 {
 	long double phi; /* Golden ratio */
 	const long double precision = 1.0e-15;
@@ -347,8 +389,8 @@ static void stress_cpu_phi(const char *name)
 	int i;
 
 	/* Pick any two starting points */
-	a = mwc() % 99;
-	b = mwc() % 99;
+	a = mwc64() % 99;
+	b = mwc64() % 99;
 
 	/* Iterate until we approach overflow */
 	for (i = 0; (i < 64) && !((a | b) & mask); i++) {
@@ -370,7 +412,7 @@ static void stress_cpu_phi(const char *name)
  *  fft_partial()
  *  	partial Fast Fourier Transform
  */
-static void fft_partial(double complex *data, double complex *tmp, const int n, const int m)
+static void HOT OPTIMIZE3 fft_partial(double complex *data, double complex *tmp, const int n, const int m)
 {
 	if (m < n) {
 		const int m2 = m * 2;
@@ -379,9 +421,10 @@ static void fft_partial(double complex *data, double complex *tmp, const int n,
 		fft_partial(tmp, data, n, m2);
 		fft_partial(tmp + m, data + m, n, m2);
 		for (i = 0; i < n; i += m2) {
+			const double complex negI = -I;
 			double complex v = tmp[i];
 			double complex t =
-				cexp((-I * M_PI * (double)i) /
+				cexp((negI * M_PI * (double)i) /
 				     (double)n) * tmp[i + m];
 			data[i / 2] = v + t;
 			data[(i + n) / 2] = v - t;
@@ -393,7 +436,7 @@ static void fft_partial(double complex *data, double complex *tmp, const int n,
  *  stress_cpu_fft()
  *	Fast Fourier Transform
  */
-static void stress_cpu_fft(const char *name)
+static void HOT stress_cpu_fft(const char *name)
 {
 	double complex buf[FFT_SIZE], tmp[FFT_SIZE];
 	int i;
@@ -411,7 +454,7 @@ static void stress_cpu_fft(const char *name)
  *   stress_cpu_euler()
  *	compute e using series
  */
-static void stress_cpu_euler(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_euler(const char *name)
 {
 	long double e = 1.0, last_e = e;
 	long double fact = 1.0;
@@ -439,7 +482,7 @@ static void random_buffer(uint8_t *data, const size_t len)
 	size_t i;
 
 	for (i = 0; i < len / 4; i++) {
-		uint32_t v = (uint32_t)mwc();
+		uint32_t v = mwc32();
 
 		*data++ = v;
 		v >>= 8;
@@ -486,7 +529,7 @@ static void stress_cpu_hash_generic(
  *	Jenkin's hash on random data
  *	http://www.burtleburtle.net/bob/hash/doobs.html
  */
-static uint32_t jenkin(const uint8_t *data, const size_t len)
+static uint32_t HOT OPTIMIZE3 jenkin(const uint8_t *data, const size_t len)
 {
 	uint8_t i;
 	register uint32_t h = 0;
@@ -527,7 +570,7 @@ static void stress_cpu_jenkin(const char *name)
  *  pjw()
  *	Hash a string, from Aho, Sethi, Ullman, Compiling Techniques.
  */
-static uint32_t pjw(const char *str)
+static uint32_t HOT OPTIMIZE3 pjw(const char *str)
 {
 	uint32_t h = 0;
 
@@ -556,7 +599,7 @@ static void stress_cpu_pjw(const char *name)
  *  djb2a()
  *	Hash a string, from Dan Bernstein comp.lang.c (xor version)
  */
-static uint32_t djb2a(const char *str)
+static uint32_t HOT OPTIMIZE3 djb2a(const char *str)
 {
 	uint32_t hash = 5381;
 	int c;
@@ -581,7 +624,7 @@ static void stress_cpu_djb2a(const char *name)
  *  fnv1a()
  *	Hash a string, using the improved 32 bit FNV-1a hash
  */
-static uint32_t fnv1a(const char *str)
+static uint32_t HOT OPTIMIZE3 fnv1a(const char *str)
 {
 	uint32_t hash = 5381;
 	const uint32_t fnv_prime = 16777619; /* 2^24 + 2^9 + 0x93 */
@@ -598,7 +641,7 @@ static uint32_t fnv1a(const char *str)
  *  stress_cpu_fnv1a()
  *	stress test hash fnv1a
  */
-static void stress_cpu_fnv1a(const char *name)
+static void HOT stress_cpu_fnv1a(const char *name)
 {
 	stress_cpu_hash_generic(name, "fnv1a", fnv1a, 0x8ef17e80);
 }
@@ -608,7 +651,7 @@ static void stress_cpu_fnv1a(const char *name)
  *	Hash a string, using the sdbm data base hash and also
  *	apparently used in GNU awk.
  */
-static uint32_t sdbm(const char *str)
+static uint32_t OPTIMIZE3 sdbm(const char *str)
 {
 	uint32_t hash = 0;
 	int c;
@@ -631,13 +674,13 @@ static void stress_cpu_sdbm(const char *name)
  *  stress_cpu_idct()
  *	compute 8x8 Inverse Discrete Cosine Transform
  */
-static void stress_cpu_idct(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_idct(const char *name)
 {
 	const double invsqrt2 = 1.0 / sqrt(2.0);
 	const double pi_over_16 = M_PI / 16.0;
 	const int sz = 8;
 	int i, j, u, v;
-	float data[sz][sz], idct[sz][sz];
+	double data[sz][sz], idct[sz][sz];
 
 	/*
 	 *  Set up DCT
@@ -702,10 +745,10 @@ static void stress_cpu_idct(const char *name)
 		b <<= 2;		\
 		a |= 1;			\
 		b |= 3;			\
-		a *= mwc();		\
-		b ^= mwc();		\
-		a += mwc();		\
-		b -= mwc();		\
+		a *= mwc32();		\
+		b ^= mwc32();		\
+		a += mwc32();		\
+		b -= mwc32();		\
 		a /= 7;			\
 		b /= 9;			\
 		a |= (c2);		\
@@ -720,7 +763,7 @@ static void stress_cpu_idct(const char *name)
  *  Generic int stressor macro
  */
 #define stress_cpu_int(_type, _sz, _a, _b, _c1, _c2, _c3)	\
-static void stress_cpu_int ## _sz(const char *name)		\
+static void HOT OPTIMIZE3 stress_cpu_int ## _sz(const char *name)\
 {								\
 	const _type mask = ~0;					\
 	const _type a_final = _a;				\
@@ -732,8 +775,8 @@ static void stress_cpu_int ## _sz(const char *name)		\
 	int i;							\
 								\
 	MWC_SEED();						\
-	a = mwc();						\
-	b = mwc();						\
+	a = mwc32();						\
+	b = mwc32();						\
 								\
 	for (i = 0; i < 1000; i++) {				\
 		int_ops(a, b, c1, c2, c3)			\
@@ -752,13 +795,13 @@ static void stress_cpu_int ## _sz(const char *name)		\
 #define _UINT128(hi, lo)	((((__uint128_t)hi << 64) | (__uint128_t)lo))
 
 stress_cpu_int(__uint128_t, 128,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x62f086e6160e4e,0xd84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3))
 #endif
 
 stress_cpu_int(uint64_t, 64, \
-	0x1ee5773113afd25aULL, 0x174df454b030714cULL,
+	0x013f7f6dc1d79197cULL, 0x01863d2c6969a51ceULL,
 	C1, C2, C3)
 
 stress_cpu_int(uint32_t, 32, \
@@ -797,10 +840,10 @@ stress_cpu_int(uint8_t, 8, \
  *  Generic floating point stressor macro
  */
 #define stress_cpu_fp(_type, _name, _sin, _cos)		\
-static void stress_cpu_ ## _name(const char *name)	\
+static void HOT OPTIMIZE3 stress_cpu_ ## _name(const char *name)\
 {							\
 	int i;						\
-	_type a = 0.18728, b = mwc(), c = mwc(), d;	\
+	_type a = 0.18728, b = mwc32(), c = mwc32(), d;	\
 							\
 	(void)name;					\
 							\
@@ -820,16 +863,21 @@ stress_cpu_fp(_Decimal64, decimal64, sin, cos)
 stress_cpu_fp(_Decimal128, decimal128, sinl, cosl)
 #endif
 
+/* Append floating point literal specifier to literal value */
+#define FP(val, ltype)	val ## ltype
+
 /*
  *  Generic complex stressor macro
  */
-#define stress_cpu_complex(_type, _name, _csin, _ccos)	\
-static void stress_cpu_ ## _name(const char *name)	\
+#define stress_cpu_complex(_type, _ltype, _name, _csin, _ccos)	\
+static void HOT OPTIMIZE3 stress_cpu_ ## _name(const char *name)\
 {							\
 	int i;						\
-	_type a = 0.18728 + I * 0.2762,			\
-		b = mwc() - I * 0.11121,		\
-		c = mwc() + I * mwc(), d;		\
+	_type cI = I;					\
+	_type a = FP(0.18728, _ltype) + 		\
+		cI * FP(0.2762, _ltype),		\
+		b = mwc32() - cI * FP(0.11121, _ltype),	\
+		c = mwc32() + cI * mwc32(), d;		\
 							\
 	(void)name;					\
 							\
@@ -840,9 +888,9 @@ static void stress_cpu_ ## _name(const char *name)	\
 	double_put(a + b + c + d);			\
 }
 
-stress_cpu_complex(complex float, complex_float, csinf, ccosf)
-stress_cpu_complex(complex double, complex_double, csin, ccos)
-stress_cpu_complex(complex long double, complex_long_double, csinl, ccosl)
+stress_cpu_complex(complex float, f, complex_float, csinf, ccosf)
+stress_cpu_complex(complex double, , complex_double, csin, ccos)
+stress_cpu_complex(complex long double, l, complex_long_double, csinl, ccosl)
 
 #define int_float_ops(_ftype, flt_a, flt_b, flt_c, flt_d,	\
 	_sin, _cos, int_a, int_b, _c1, _c2, _c3)		\
@@ -872,12 +920,12 @@ stress_cpu_complex(complex long double, complex_long_double, csinl, ccosl)
 		int_a |= 1;					\
 		flt_d = flt_d + flt_b + (_ftype)_sin(flt_a);	\
 		int_b |= 3;					\
-		int_a *= mwc();					\
+		int_a *= mwc32();				\
 		flt_a = (flt_b + flt_c) / flt_c;		\
-		int_b ^= mwc();					\
-		int_a += mwc();					\
+		int_b ^= mwc32();				\
+		int_a += mwc32();				\
 		flt_b = flt_b * flt_c;				\
-		int_b -= mwc();					\
+		int_b -= mwc32();				\
 		int_a /= 7;					\
 		flt_c = flt_c + (_ftype)1.0;			\
 		int_b /= 9;					\
@@ -896,7 +944,7 @@ stress_cpu_complex(complex long double, complex_long_double, csinl, ccosl)
  */
 #define stress_cpu_int_fp(_inttype, _sz, _ftype, _name, _a, _b, \
 	_c1, _c2, _c3, _sinf, _cosf)				\
-static void stress_cpu_int ## _sz ## _ ## _name(const char *name)\
+static void HOT OPTIMIZE3 stress_cpu_int ## _sz ## _ ## _name(const char *name)\
 {								\
 	int i;							\
 	_inttype int_a, int_b;					\
@@ -906,12 +954,12 @@ static void stress_cpu_int ## _sz ## _ ## _name(const char *name)\
 	const _inttype c1 = _c1 & mask;				\
 	const _inttype c2 = _c2 & mask;				\
 	const _inttype c3 = _c3 & mask;				\
-	_ftype flt_a = 0.18728, flt_b = mwc(), 			\
-		flt_c = mwc(), flt_d;				\
+	_ftype flt_a = 0.18728, flt_b = mwc32(),		\
+		flt_c = mwc32(), flt_d;				\
 								\
 	MWC_SEED();						\
-	int_a = mwc();						\
-	int_b = mwc();						\
+	int_a = mwc32();					\
+	int_b = mwc32();					\
 								\
 	for (i = 0; i < 1000; i++) {				\
 		int_float_ops(_ftype, flt_a, flt_b, flt_c, flt_d,\
@@ -920,7 +968,7 @@ static void stress_cpu_int ## _sz ## _ ## _name(const char *name)\
 	if ((opt_flags & OPT_FLAGS_VERIFY) &&			\
 	    ((int_a != a_final) || (int_b != b_final)))		\
 		pr_fail(stderr, "%s: int" # _sz " error detected, "\
-			"failed int" # _sz 			\
+			"failed int" # _sz "" # _ftype		\
 			" math operations\n", name);		\
 								\
 	double_put(flt_a + flt_b + flt_c + flt_d);		\
@@ -936,45 +984,45 @@ stress_cpu_int_fp(uint32_t, 32, long double, longdouble,
 	0x1ce9b547UL, 0xa24b33aUL,
 	C1, C2, C3, sinl, cosl)
 stress_cpu_int_fp(uint64_t, 64, float, float,
-	0x1ee5773113afd25aULL, 0x174df454b030714cULL,
+	0x13f7f6dc1d79197cULL, 0x1863d2c6969a51ceULL,
 	C1, C2, C3, sinf, cosf)
 stress_cpu_int_fp(uint64_t, 64, double, double,
-	0x1ee5773113afd25aULL, 0x174df454b030714cULL,
+	0x13f7f6dc1d79197cULL, 0x1863d2c6969a51ceULL,
 	C1, C2, C3, sin, cos)
 stress_cpu_int_fp(uint64_t, 64, long double, longdouble,
-	0x1ee5773113afd25aULL, 0x174df454b030714cULL,
+	0x13f7f6dc1d79197cULL, 0x1863d2c6969a51ceULL,
 	C1, C2, C3, sinl, cosl)
 
 #if defined(STRESS_INT128)
 stress_cpu_int_fp(__uint128_t, 128, float, float,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	sinf, cosf)
 stress_cpu_int_fp(__uint128_t, 128, double, double,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	sin, cos)
 stress_cpu_int_fp(__uint128_t, 128, long double, longdouble,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	sinl, cosl)
 #if defined(STRESS_FLOAT_DECIMAL)
 stress_cpu_int_fp(__uint128_t, 128, _Decimal32, decimal32,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	(_Decimal32)sinf, (_Decimal32)cosf)
 stress_cpu_int_fp(__uint128_t, 128, _Decimal64, decimal64,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	(_Decimal64)sin, (_Decimal64)cos)
 stress_cpu_int_fp(__uint128_t, 128, _Decimal128, decimal128,
-	_UINT128(0x1caaffe276809a64,0xf7a3387557025785),
-	_UINT128(0x052970104c342020,0x4e4cc51e06b44800),
+	_UINT128(0x132af604d8b9183a,0x5e3af8fa7a663d74),
+	_UINT128(0x0062f086e6160e4e,0x0d84c9f800365858),
 	_UINT128(C1, C1), _UINT128(C2, C2), _UINT128(C3, C3),
 	(_Decimal128)sinl, (_Decimal128)cosl)
 #endif
@@ -984,10 +1032,10 @@ stress_cpu_int_fp(__uint128_t, 128, _Decimal128, decimal128,
  *  stress_cpu_rgb()
  *	CCIR 601 RGB to YUV to RGB conversion
  */
-static void stress_cpu_rgb(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_rgb(const char *name)
 {
 	int i;
-	uint32_t rgb = mwc() & 0xffffff;
+	uint32_t rgb = mwc32() & 0xffffff;
 	uint8_t r = rgb >> 16;
 	uint8_t g = rgb >> 8;
 	uint8_t b = rgb;
@@ -999,28 +1047,28 @@ static void stress_cpu_rgb(const char *name)
 		float y,u,v;
 
 		/* RGB to CCIR 601 YUV */
-		y = (0.299 * r) + (0.587 * g) + (0.114 * b);
-		u = (b - y) * 0.565;
-		v = (r - y) * 0.713;
+		y = (0.299f * r) + (0.587f * g) + (0.114f * b);
+		u = (b - y) * 0.565f;
+		v = (r - y) * 0.713f;
 
 		/* YUV back to RGB */
-		r = y + (1.403 * v);
-		g = y - (0.344 * u) - (0.714 * v);
-		b = y + (1.770 * u);
+		r = y + (1.403f * v);
+		g = y - (0.344f * u) - (0.714f * v);
+		b = y + (1.770f * u);
 
 		/* And bump each colour to make next round */
 		r += 1;
 		g += 2;
 		b += 3;
+		uint64_put(r + g + b);
 	}
-	uint64_put(r + g + b);
 }
 
 /*
  *  stress_cpu_matrix_prod(void)
  *	matrix product
  */
-static void stress_cpu_matrix_prod(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_matrix_prod(const char *name)
 {
 	int i, j, k;
 	const int n = 128;
@@ -1033,8 +1081,8 @@ static void stress_cpu_matrix_prod(const char *name)
 
 	for (i = 0; i < n; i++) {
 		for (j = 0; j < n; j++) {
-			a[i][j] = (long double)mwc() * v;
-			b[i][j] = (long double)mwc() * v;
+			a[i][j] = (long double)mwc32() * v;
+			b[i][j] = (long double)mwc32() * v;
 			r[i][j] = 0.0;
 		}
 	}
@@ -1057,7 +1105,7 @@ static void stress_cpu_matrix_prod(const char *name)
  *   stress_cpu_fibonacci()
  *	compute fibonacci series
  */
-static void stress_cpu_fibonacci(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_fibonacci(const char *name)
 {
 	const uint64_t fn_res = 0xa94fad42221f2702ULL;
 	register uint64_t f1 = 0, f2 = 1, fn;
@@ -1077,7 +1125,7 @@ static void stress_cpu_fibonacci(const char *name)
  *	compute the constant psi,
  * 	the reciprocal Fibonacci constant
  */
-static void stress_cpu_psi(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_psi(const char *name)
 {
 	long double f1 = 0.0, f2 = 1.0;
 	long double psi = 0.0, last_psi;
@@ -1108,7 +1156,7 @@ static void stress_cpu_psi(const char *name)
  *   stress_cpu_ln2
  *	compute ln(2) using series
  */
-static void stress_cpu_ln2(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_ln2(const char *name)
 {
 	long double ln2 = 0.0, last_ln2 = 0.0;
 	long double precision = 1.0e-7;
@@ -1139,7 +1187,7 @@ static void stress_cpu_ln2(const char *name)
  *  ackermann()
  *	a naive/simple implementation of the ackermann function
  */
-static uint32_t ackermann(const uint32_t m, const uint32_t n)
+static uint32_t HOT ackermann(const uint32_t m, const uint32_t n)
 {
 	if (m == 0)
 		return n + 1;
@@ -1165,7 +1213,7 @@ static void stress_cpu_ackermann(const char *name)
  *   stress_cpu_explog
  *	compute exp(log(n))
  */
-static void stress_cpu_explog(const char *name)
+static void HOT stress_cpu_explog(const char *name)
 {
 	uint32_t i;
 	double n = 1e6;
@@ -1198,7 +1246,7 @@ static void stress_cpu_jmp(const char *name)  __attribute__((optimize("-O0")));
  *   stress_cpu_jmp
  *	jmp conditionals
  */
-static void stress_cpu_jmp(const char *name)
+static void HOT stress_cpu_jmp(const char *name)
 {
 	register int i, next = 0;
 
@@ -1226,7 +1274,7 @@ static void stress_cpu_jmp(const char *name)
  *  ccitt_crc16()
  *	perform naive CCITT CRC16
  */
-static uint16_t ccitt_crc16(const uint8_t *data, size_t n)
+static uint16_t HOT OPTIMIZE3 ccitt_crc16(const uint8_t *data, size_t n)
 {
 	/*
 	 *  The CCITT CRC16 polynomial is
@@ -1281,7 +1329,7 @@ static void stress_cpu_crc16(const char *name)
  *  zeta()
  *	Riemann zeta function
  */
-static inline long double complex zeta(
+static inline long double complex HOT OPTIMIZE3 zeta(
 	const long double complex s,
 	long double precision)
 {
@@ -1315,7 +1363,7 @@ static void stress_cpu_zeta(const char *name)
  * stress_cpu_gamma()
  *	stress Euler–Mascheroni constant gamma
  */
-static void stress_cpu_gamma(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_gamma(const char *name)
 {
 	long double precision = 1.0e-10;
 	long double sum = 0.0, k = 1.0, gamma = 0.0, gammaold;
@@ -1344,7 +1392,7 @@ static void stress_cpu_gamma(const char *name)
  *  Introduction to Signal Processing,
  *  Prentice-Hall, 1995, ISBN: 0-13-209172-0.
  */
-static void stress_cpu_correlate(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_correlate(const char *name)
 {
 	const size_t data_len = 16384;
 	const size_t corr_len = data_len / 16;
@@ -1356,7 +1404,7 @@ static void stress_cpu_correlate(const char *name)
 
 	/* Generate some random data */
 	for (i = 0; i < data_len; i++) {
-		data[i] = mwc();
+		data[i] = mwc64();
 		data_average += data[i];
 	}
 	data_average /= (double)data_len;
@@ -1378,7 +1426,7 @@ static void stress_cpu_correlate(const char *name)
  * stress_cpu_sieve()
  * 	slightly optimised Sieve of Eratosthenes
  */
-static void stress_cpu_sieve(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_sieve(const char *name)
 {
 	const uint32_t nsqrt = sqrt(SIEVE_SIZE);
 	uint32_t sieve[(SIEVE_SIZE + 31) / 32];
@@ -1386,13 +1434,13 @@ static void stress_cpu_sieve(const char *name)
 
 	memset(sieve, 0xff, sizeof(sieve));
 	for (i = 2; i < nsqrt; i++)
-		if (SIEVE_GETBIT(sieve, i))
+		if (STRESS_GETBIT(sieve, i))
 			for (j = i * i; j < SIEVE_SIZE; j += i)
-				SIEVE_CLRBIT(sieve, j);
+				STRESS_CLRBIT(sieve, j);
 
 	/* And count up number of primes */
 	for (j = 0, i = 2; i < SIEVE_SIZE; i++) {
-		if (SIEVE_GETBIT(sieve, i))
+		if (STRESS_GETBIT(sieve, i))
 			j++;
 	}
 	if ((opt_flags & OPT_FLAGS_VERIFY) && (j != 664579))
@@ -1404,7 +1452,7 @@ static void stress_cpu_sieve(const char *name)
  *	return true if n is prime
  *	http://en.wikipedia.org/wiki/Primality_test
  */
-static inline bool is_prime(uint32_t n)
+static inline HOT OPTIMIZE3 bool is_prime(uint32_t n)
 {
 	uint32_t i, max;
 
@@ -1440,21 +1488,35 @@ static void stress_cpu_prime(const char *name)
  *  stress_cpu_gray()
  *	compute gray codes
  */
-static void stress_cpu_gray(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_gray(const char *name)
 {
 	uint32_t i;
 	uint64_t sum = 0;
 
 	for (i = 0; i < 0x10000; i++) {
-		register uint32_t gray_code, mask;
+		register uint32_t gray_code;
 
 		/* Binary to Gray code */
 		gray_code = (i >> 1) ^ i;
 		sum += gray_code;
 
 		/* Gray code back to binary */
-		for (mask = gray_code >> 1; mask; mask >>= 1)
-			gray_code ^= mask;
+#if 0
+		{
+			/* Slow iterative method */
+			register uint32_t mask;
+
+			for (mask = gray_code >> 1; mask; mask >>= 1)
+				gray_code ^= mask;
+		}
+#else
+		/* Fast non-loop method */
+		gray_code ^= (gray_code >> 1);
+		gray_code ^= (gray_code >> 2);
+		gray_code ^= (gray_code >> 4);
+		gray_code ^= (gray_code >> 8);
+		gray_code ^= (gray_code >> 16);
+#endif
 		sum += gray_code;
 	}
 	if ((opt_flags & OPT_FLAGS_VERIFY) && (sum != 0xffff0000))
@@ -1466,7 +1528,7 @@ static void stress_cpu_gray(const char *name)
  * hanoi()
  *	do a Hanoi move
  */
-static uint32_t hanoi(
+static uint32_t HOT hanoi(
 	const uint16_t n,
 	const char p1,
 	const char p2,
@@ -1501,7 +1563,7 @@ static void stress_cpu_hanoi(const char *name)
  *  factorial()
  *	compute n!
  */
-static long double factorial(int n)
+static long double HOT OPTIMIZE3 factorial(int n)
 {
 	long double f = 1;
 
@@ -1517,7 +1579,7 @@ static long double factorial(int n)
  *	compute pi using the Srinivasa Ramanujan
  *	fast convergence algorithm
  */
-static void stress_cpu_pi(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_pi(const char *name)
 {
 	long double s = 0.0, pi = 0.0, last_pi = 0.0;
 	const long double precision = 1.0e-20;
@@ -1550,7 +1612,7 @@ static void stress_cpu_pi(const char *name)
  *	compute the constant omega
  *	See http://en.wikipedia.org/wiki/Omega_constant
  */
-static void stress_cpu_omega(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_omega(const char *name)
 {
 	long double omega = 0.5, last_omega = 0.0;
 	const long double precision = 1.0e-20;
@@ -1588,7 +1650,7 @@ static void stress_cpu_omega(const char *name)
  *  hamming84()
  *	compute Hamming (8,4) codes
  */
-static uint8_t hamming84(const uint8_t nybble)
+static uint8_t HOT OPTIMIZE3 hamming84(const uint8_t nybble)
 {
 	/*
 	 * Hamming (8,4) Generator matrix
@@ -1636,7 +1698,7 @@ static uint8_t hamming84(const uint8_t nybble)
  *  stress_cpu_hamming()
  *	compute hamming code on 65536 x 4 nybbles
  */
-static void stress_cpu_hamming(const char *name)
+static void HOT OPTIMIZE3 stress_cpu_hamming(const char *name)
 {
 	uint32_t i;
 	uint32_t sum = 0;
@@ -1657,11 +1719,328 @@ static void stress_cpu_hamming(const char *name)
 		pr_fail(stderr, "%s: hamming error detected, sum of 65536 hamming codes not correct\n", name);
 }
 
+
+static ptrdiff_t stress_cpu_callfunc_func(
+	ssize_t		n,
+	uint64_t	u64arg,
+	uint32_t	u32arg,
+	uint16_t	u16arg,
+	uint8_t		u8arg,
+	uint64_t	*p_u64arg,
+	uint32_t	*p_u32arg,
+	uint16_t	*p_u16arg,
+	uint8_t		*p_u8arg)
+{
+	if (n > 0)
+		return stress_cpu_callfunc_func(n - 1,
+			u64arg, u32arg, u16arg, u8arg,
+			p_u64arg, p_u32arg, p_u16arg, p_u8arg);
+	else
+		return &u64arg - p_u64arg;
+}
+
+/*
+ *  stress_cpu_callfunc()
+ *	deep function calls
+ */
+static void stress_cpu_callfunc(const char *name)
+{
+	uint64_t	u64arg = mwc64();
+	uint32_t	u32arg = mwc32();
+	uint16_t	u16arg = mwc16();
+	uint8_t		u8arg  = mwc8();
+	ptrdiff_t	ret;
+
+	(void)name;
+
+	ret = stress_cpu_callfunc_func(1024,
+		u64arg, u32arg, u16arg, u8arg,
+		&u64arg, &u32arg, &u16arg, &u8arg);
+
+	uint64_put((uint64_t)ret);
+}
+
+
+#define P2(n) n, n^1, n^1, n
+#define P4(n) P2(n), P2(n^1), P2(n^1), P2(n)
+#define P6(n) P4(n), P4(n^1), P4(n^1), P4(n)
+
+static const bool stress_cpu_parity_table[256] = {
+	P6(0), P6(1), P6(1), P6(0)
+};
+
+/*
+ *  stress_cpu_parity
+ *	compute parity different ways
+ */
+static void stress_cpu_parity(const char *name)
+{
+	uint32_t val = 0x83fb5acf;
+	size_t i;
+
+	for (i = 0; i < 1000; i++, val++) {
+		uint32_t v, parity, p;
+		uint8_t *ptr;
+
+		/*
+		 * Naive way
+		 */
+		v = val;
+		parity = 0;
+		while (v) {
+			if (v & 1)
+				parity = !parity;
+			v >>= 1;
+		}
+
+		/*
+		 * Naive way with Brian Kernigan's bit counting optimisation
+		 * https://graphics.stanford.edu/~seander/bithacks.html
+		 */
+		v = val;
+		p = 0;
+		while (v) {
+			p = !p;
+			v = v & (v - 1);
+		}
+		if ((opt_flags & OPT_FLAGS_VERIFY) && (p != parity))
+			pr_fail(stderr, "%s: parity error detected, using optimised naive method\n",  name);
+
+		/*
+		 * "Compute parity of a word with a multiply"
+		 * the Andrew Shapira method,
+		 * https://graphics.stanford.edu/~seander/bithacks.html
+		 */
+		v = val;
+		v ^= v >> 1;
+		v ^= v >> 2;
+		v = (v & 0x11111111U) * 0x11111111U;
+		p = (v >> 28) & 1;
+		if ((opt_flags & OPT_FLAGS_VERIFY) && (p != parity))
+			pr_fail(stderr, "%s: parity error detected, using the multiply Shapira method\n",  name);
+
+		/*
+		 * "Compute parity in parallel"
+		 * https://graphics.stanford.edu/~seander/bithacks.html
+		 */
+		v = val;
+		v ^= v >> 16;
+		v ^= v >> 8;
+		v ^= v >> 4;
+		v &= 0xf;
+		p = (0x6996 >> v) & 1;
+		if ((opt_flags & OPT_FLAGS_VERIFY) && (p != parity))
+			pr_fail(stderr, "%s: parity error detected, using the parallel method\n",  name);
+
+		/*
+		 * "Compute parity by lookup table"
+		 * https://graphics.stanford.edu/~seander/bithacks.html
+		 * Variation #1
+		 */
+		v = val;
+		v ^= v >> 16;
+		v ^= v >> 8;
+		p = stress_cpu_parity_table[v & 0xff];
+		if ((opt_flags & OPT_FLAGS_VERIFY) && (p != parity))
+			pr_fail(stderr, "%s: parity error detected, using the lookup method, variation 1\n",  name);
+
+		/*
+		 * "Compute parity by lookup table"
+		 * https://graphics.stanford.edu/~seander/bithacks.html
+		 * Variation #2
+		 */
+		ptr = (uint8_t *)&val;
+		p = stress_cpu_parity_table[ptr[0] ^ ptr[1] ^ ptr[2] ^ ptr[3]];
+		if ((opt_flags & OPT_FLAGS_VERIFY) && (p != parity))
+			pr_fail(stderr, "%s: parity error detected, using the lookup method, variation 1\n",  name);
+	}
+}
+
+/*
+ *  stress_cpu_dither
+ *	perform 8 bit to 1 bit gray scale
+ *	Floyd–Steinberg dither
+ */
+static void stress_cpu_dither(const char *name)
+{
+	size_t x, y;
+
+	(void)name;
+
+	/*
+	 *  Generate some random 8 bit image
+	 */
+	for (y = 0; y < STRESS_CPU_DITHER_Y; y += 8) {
+		for (x = 0; x < STRESS_CPU_DITHER_X; x ++) {
+			uint64_t v = mwc64();
+
+			pixels[x][y + 0] = v;
+			v >>= 8;
+			pixels[x][y + 1] = v;
+			v >>= 8;
+			pixels[x][y + 2] = v;
+			v >>= 8;
+			pixels[x][y + 3] = v;
+			v >>= 8;
+			pixels[x][y + 4] = v;
+			v >>= 8;
+			pixels[x][y + 5] = v;
+			v >>= 8;
+			pixels[x][y + 6] = v;
+			v >>= 8;
+			pixels[x][y + 7] = v;
+		}
+	}
+
+	/*
+	 *  ..and dither
+	 */
+	for (y = 0; y < STRESS_CPU_DITHER_Y; y++) {
+		for (x = 0; x < STRESS_CPU_DITHER_X; x++) {
+			uint8_t pixel = pixels[x][y];
+			uint8_t quant = (pixel < 128) ? 0 : 255;
+			int32_t error = pixel - quant;
+
+			bool xok1 = x < (STRESS_CPU_DITHER_X - 1);
+			bool xok2 = x > 0;
+			bool yok1 = y < (STRESS_CPU_DITHER_Y - 1);
+
+			if (xok1)
+				pixels[x + 1][y] +=
+					(error * 7) >> 4;
+			if (xok2 && yok1)
+				pixels[x - 1][y + 1] +=
+					(error * 3) >> 4;
+			if (yok1)
+				pixels[x][y + 1] +=
+					(error * 5) >> 4;
+			if (xok1 && yok1)
+				pixels[x + 1][y + 1] +=
+					error >> 4;
+		}
+	}
+}
+
+/*
+ *  stress_cpu_union
+ *	perform bit field operations on a packed union
+ */
+static void stress_cpu_union(const char *name)
+{
+	typedef union {
+		struct {
+			uint64_t	b1:1;
+			uint64_t	b10:10;
+			uint64_t	b2:2;
+			uint64_t	b9:9;
+			uint64_t	b3:3;
+			uint64_t	b8:8;
+			uint64_t	b4:4;
+			uint64_t	b7:7;
+			uint64_t	b5:5;
+			uint64_t	b6:6;
+		} bits64;
+		uint64_t	u64:64;
+		union {
+			uint8_t		b1:1;
+			uint8_t		b7:7;
+			uint8_t		b8:8;
+		} bits8;
+		struct {
+			uint16_t	b15:15;
+			uint16_t	b1:1;
+		} bits16;
+		struct {
+			uint32_t	b10:10;
+			uint32_t	b20:20;
+			uint32_t	:1;
+			uint32_t	b1:1;
+		} bits32;
+		uint32_t	u32:30;
+	} __attribute__ ((packed)) u_t;
+
+	static u_t u;
+	size_t i;
+
+	(void)name;
+	for (i = 0; i < 1000; i++) {
+		u.bits64.b1 ^= 1;
+		u.bits64.b2--;
+		u.bits32.b10 ^= ~0;
+		u.bits64.b3++;
+		u.bits16.b1--;
+		u.bits8.b1++;
+		u.bits64.b4 *= 2;
+		u.bits32.b20 += 3;
+		u.u64 += 0x1037fc2ae21ef829ULL;
+		u.bits64.b6--;
+		u.bits8.b7 *= 3;
+		u.bits64.b5 += (u.bits64.b4 << 1);
+		u.bits32.b1 ^= 1;
+		u.bits64.b7++;
+		u.bits8.b8 ^= 0xaa;
+		u.bits64.b8--;
+		u.bits16.b15 ^= 0xbeef;
+		u.bits64.b9++;
+		u.bits64.b10 *= 5;
+	}
+}
+
+static uint32_t queens_solutions[] = {
+	-1, 1, 0, 0, 2, 10, 4, 40, 92, 352, 724, 2680, 14200
+};
+
+/*
+ *  Solution from http://www.cl.cam.ac.uk/~mr10/backtrk.pdf
+ *     see section 2.1
+ */
+static uint32_t queens_try(
+	uint32_t left_diag,
+	uint32_t cols,
+	uint32_t right_diag,
+	uint32_t all)
+{
+	register uint32_t solutions = 0;
+	register uint32_t poss = ~(left_diag | cols | right_diag) & all;
+
+	while (poss) {
+		register uint32_t bit = poss & -poss;
+		register uint32_t new_cols = cols | bit;
+
+		poss -= bit;
+		solutions += (new_cols == all) ?
+			1 : queens_try((left_diag | bit) << 1,
+				new_cols, (right_diag | bit) >> 1, all);
+	}
+	return solutions;
+}
+
+
+/*
+ *  stress_cpu_queens
+ *	solve the queens problem for sizes 1..12
+ */
+static void stress_cpu_queens(const char *name)
+{
+	uint32_t all, n;
+
+        for (all = 1, n = 1; n < 13; n++) {
+		uint32_t solutions = queens_try(0, 0, 0, all);
+		if ((opt_flags & OPT_FLAGS_VERIFY) && 
+		    (solutions != queens_solutions[n]))
+			pr_fail(stderr, "%s: queens solution error detected "
+				"on board size %" PRIu32 "\n", 
+				name, n);
+                all = (all + all) + 1;
+        }
+
+}
+
 /*
  *  stress_cpu_all()
  *	iterate over all cpu stressors
  */
-static void stress_cpu_all(const char *name)
+static HOT OPTIMIZE3 void stress_cpu_all(const char *name)
 {
 	static int i = 1;	/* Skip over stress_cpu_all */
 
@@ -1678,24 +2057,26 @@ static stress_cpu_stressor_info_t cpu_methods[] = {
 
 	{ "ackermann",		stress_cpu_ackermann },
 	{ "bitops",		stress_cpu_bitops },
-	{ "crc16",		stress_cpu_crc16 },
+	{ "callfunc",		stress_cpu_callfunc },
 	{ "cdouble",		stress_cpu_complex_double },
 	{ "cfloat",		stress_cpu_complex_float },
 	{ "clongdouble",	stress_cpu_complex_long_double },
 	{ "correlate",		stress_cpu_correlate },
+	{ "crc16",		stress_cpu_crc16 },
 #if defined(STRESS_FLOAT_DECIMAL)
 	{ "decimal32",		stress_cpu_decimal32 },
 	{ "decimal64",		stress_cpu_decimal64 },
 	{ "decimal128",		stress_cpu_decimal128 },
 #endif
-	{ "double",		stress_cpu_double },
+	{ "dither",		stress_cpu_dither },
 	{ "djb2a",		stress_cpu_djb2a },
+	{ "double",		stress_cpu_double },
 	{ "euler",		stress_cpu_euler },
 	{ "explog",		stress_cpu_explog },
-	{ "fibonacci",		stress_cpu_fibonacci },
-	{ "fnv1a",		stress_cpu_fnv1a },
 	{ "fft",		stress_cpu_fft },
+	{ "fibonacci",		stress_cpu_fibonacci },
 	{ "float",		stress_cpu_float },
+	{ "fnv1a",		stress_cpu_fnv1a },
 	{ "gamma",		stress_cpu_gamma },
 	{ "gcd",		stress_cpu_gcd },
 	{ "gray",		stress_cpu_gray },
@@ -1734,17 +2115,21 @@ static stress_cpu_stressor_info_t cpu_methods[] = {
 	{ "matrixprod",		stress_cpu_matrix_prod },
 	{ "nsqrt",		stress_cpu_nsqrt },
 	{ "omega",		stress_cpu_omega },
+	{ "parity",		stress_cpu_parity },
 	{ "phi",		stress_cpu_phi },
 	{ "pi",			stress_cpu_pi },
 	{ "pjw",		stress_cpu_pjw },
 	{ "prime",		stress_cpu_prime },
 	{ "psi",		stress_cpu_psi },
+	{ "queens",		stress_cpu_queens },
 	{ "rand",		stress_cpu_rand },
+	{ "rand48",		stress_cpu_rand48 },
 	{ "rgb",		stress_cpu_rgb },
 	{ "sdbm",		stress_cpu_sdbm },
 	{ "sieve",		stress_cpu_sieve },
 	{ "sqrt", 		stress_cpu_sqrt },
 	{ "trig",		stress_cpu_trig },
+	{ "union",		stress_cpu_union },
 	{ "zeta",		stress_cpu_zeta },
 	{ NULL,			NULL }
 };
@@ -1753,7 +2138,7 @@ static stress_cpu_stressor_info_t cpu_methods[] = {
  *  stress_set_cpu_method()
  *	set the default cpu stress method
  */
-int stress_set_cpu_method(const char *name)
+int HOT OPTIMIZE3 stress_set_cpu_method(const char *name)
 {
 	stress_cpu_stressor_info_t *info = cpu_methods;
 
@@ -1816,29 +2201,49 @@ int stress_cpu(
 	 */
 	bias = 0.0;
 	do {
-		int j;
 		double t, delay;
-		struct timeval tv1, tv2, tv3;
+		double t1, t2, t3;
+		struct timeval tv;
 
-		gettimeofday(&tv1, NULL);
-		for (j = 0; j < 64; j++) {
-			(void)func(name);
-			if (!opt_do_run)
-				break;
-			(*counter)++;
+		t1 = time_now();
+		if (opt_cpu_load_slice < 0) {
+			/* < 0, specifies number of iterations to do per slice */
+			int j;
+
+			for (j = 0; j < -opt_cpu_load_slice; j++) {
+				(void)func(name);
+				if (!opt_do_run)
+					break;
+				(*counter)++;
+			}
+			t2 = time_now();
+		} else if (opt_cpu_load_slice == 0) {
+			/* == 0, random time slices */
+			double slice_end = t1 + (((double)mwc16()) / 131072.0);
+			do {
+				(void)func(name);
+				t2 = time_now();
+			} while (t2 < slice_end);
+		} else {
+			/* > 0, time slice in milliseconds */
+			double slice_end = t1 + ((double)opt_cpu_load_slice / 1000.0);
+			do {
+				(void)func(name);
+				t2 = time_now();
+			} while (t2 < slice_end);
 		}
-		gettimeofday(&tv2, NULL);
-		t = timeval_to_double(&tv2) - timeval_to_double(&tv1);
+		t = t2 - t1;
 		/* Must not calculate this with zero % load */
 		delay = t * (((100.0 / (double) opt_cpu_load)) - 1.0);
 		delay -= bias;
 
-		tv1.tv_sec = delay;
-		tv1.tv_usec = (delay - tv1.tv_sec) * 1000000.0;
-		select(0, NULL, NULL, NULL, &tv1);
-		gettimeofday(&tv3, NULL);
+		tv.tv_sec = delay;
+		tv.tv_usec = (delay - tv.tv_sec) * 1000000.0;
+		select(0, NULL, NULL, NULL, &tv);
+
+		t3 = time_now();
 		/* Bias takes account of the time to do the delay */
-		bias = (timeval_to_double(&tv3) - timeval_to_double(&tv2)) - delay;
+		bias = (t3 - t2) - delay;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
 
 	return EXIT_SUCCESS;
