@@ -91,7 +91,7 @@ int stress_set_epoll_domain(const char *name)
 {
 	int ret;
 
-	ret = stress_set_net_domain("epoll-domain", name, &opt_epoll_domain);
+	ret = stress_set_net_domain(DOMAIN_ALL, "epoll-domain", name, &opt_epoll_domain);
 
 	switch (opt_epoll_domain) {
 	case AF_INET:
@@ -110,7 +110,7 @@ int stress_set_epoll_domain(const char *name)
  * epoll_timer_handler()
  *	catch timer signal and cancel if no more runs flagged
  */
-static void epoll_timer_handler(int sig)
+static void MLOCKED epoll_timer_handler(int sig)
 {
 	(void)sig;
 
@@ -132,7 +132,7 @@ static void epoll_timer_handler(int sig)
  *  handle_socket_sigalrm()
  *	catch SIGALRM
  */
-static void handle_socket_sigalrm(int dummy)
+static MLOCKED void handle_socket_sigalrm(int dummy)
 {
 	(void)dummy;
 	opt_do_run = false;
@@ -143,7 +143,7 @@ static void handle_socket_sigalrm(int dummy)
  *  epoll_spawn()
  *	spawn a process
  */
-static int epoll_spawn(
+static pid_t epoll_spawn(
 	epoll_func_t func,
 	const int child,
 	uint64_t *const counter,
@@ -154,8 +154,11 @@ static int epoll_spawn(
 {
 	pid_t pid;
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		return -1;
 	}
 	if (pid == 0) {
@@ -292,7 +295,7 @@ static int epoll_client(
 
 	do {
 		char buf[4096];
-		int fd;
+		int fd, saved_errno;
 		int retries = 0;
 		int ret = -1;
 		int port = opt_epoll_port + port_counter + (max_servers * instance);
@@ -333,12 +336,13 @@ retry:
 			(void)close(fd);
 			return -1;
 		}
-		errno = 0;
 
 		stress_set_sockaddr(name, instance, ppid,
 			opt_epoll_domain, port, &addr, &addr_len);
 
+		errno = 0;
 		ret = connect(fd, addr, addr_len);
+		saved_errno = errno;
 
 		/* No longer need timer */
 		if (timer_delete(epoll_timerid) < 0) {
@@ -348,7 +352,7 @@ retry:
 		}
 
 		if (ret < 0) {
-			switch (errno) {
+			switch (saved_errno) {
 			case EINTR:
 				connect_timeouts++;
 				break;
@@ -357,16 +361,17 @@ retry:
 				break;
 			default:
 				pr_dbg(stderr, "%s: connect failed: %d (%s)\n",
-					name, errno, strerror(errno));
+					name, saved_errno, strerror(saved_errno));
 				break;
 			}
 			(void)close(fd);
-			usleep(1000);		/* Twiddle fingers for a moment */
+			usleep(100000);		/* Twiddle fingers for a moment */
 
 			retries++;
-			if (retries > 100) {
+			if (retries > 1000) {
 				/* Sigh, give up.. */
-				pr_failed_dbg(name, "connect");
+				errno = saved_errno;
+				pr_failed_dbg(name, "too many connects");
 				return -1;
 			}
 			goto retry;
@@ -505,7 +510,7 @@ static void epoll_server(
 			} else if (sfd == events[i].data.fd) {
 				/*
 				 *  The listening socket has notification(s)
-				 *  pending, so handle incomming connections
+				 *  pending, so handle incoming connections
 				 */
 				if (epoll_notification(name, efd, sfd) < 0)
 					break;
