@@ -53,7 +53,7 @@ void stress_set_fifo_readers(const char *optarg)
  *  fifo_spawn()
  *	spawn a process
  */
-static int fifo_spawn(
+static pid_t fifo_spawn(
 	void (*func)(const char *name, const char *fifoname),
 	const char *name,
 	const char *fifoname)
@@ -144,7 +144,7 @@ int stress_fifo(
 		return EXIT_FAILURE;
 
 	(void)stress_temp_filename(fifoname, sizeof(fifoname),
-                name, pid, instance, mwc());
+                name, pid, instance, mwc32());
 	(void)umask(0077);
 
 	if (mkfifo(fifoname, S_IRUSR | S_IWUSR) < 0) {
