@@ -73,12 +73,15 @@ int stress_futex(
 	const uint64_t max_ops,
 	const char *name)
 {
-	uint64_t *timeout = &shared->futex_timeout[instance];
-	uint32_t *futex = &shared->futex[instance];
+	uint64_t *timeout = &shared->futex.timeout[instance];
+	uint32_t *futex = &shared->futex.futex[instance];
 	pid_t pid;
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	}
