@@ -59,7 +59,7 @@ void stress_set_lease_breakers(const char *optarg)
  *  stress_lease_handler()
  *	lease signal handler
  */
-static void stress_lease_handler(int dummy)
+static void MLOCKED stress_lease_handler(int dummy)
 {
 	(void)dummy;
 
@@ -70,7 +70,7 @@ static void stress_lease_handler(int dummy)
  *  stress_lease_spawn()
  *	spawn a process
  */
-static int stress_lease_spawn(
+static pid_t stress_lease_spawn(
 	const char *filename,
 	const char *name,
 	const uint64_t max_ops,
@@ -85,8 +85,11 @@ static int stress_lease_spawn(
 			opt_lease_breakers = MIN_LEASE_BREAKERS;
 	}
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		return -1;
 	}
 	if (pid == 0) {
@@ -97,7 +100,7 @@ static int stress_lease_spawn(
 			fd = open(filename, O_NONBLOCK | O_WRONLY, S_IRUSR | S_IWUSR);
 			if (fd < 0) {
 				if (errno != EWOULDBLOCK && errno != EACCES) {
-					pr_dbg(stderr, "%s: open failed: errno=%d: (%s)\n",
+					pr_dbg(stderr, "%s: open failed (child): errno=%d: (%s)\n",
 						name, errno, strerror(errno));
 				}
 				continue;
@@ -141,11 +144,11 @@ int stress_lease(
 	if (stress_temp_dir_mk(name, pid, instance) < 0)
 		return EXIT_FAILURE;
 	(void)stress_temp_filename(filename, PATH_MAX,
-		name, pid, instance, mwc());
+		name, pid, instance, mwc32());
 
 	fd = creat(filename, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
-		pr_err(stderr, "%s: open failed: errno=%d: (%s)\n",
+		pr_err(stderr, "%s: creat failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 		(void)stress_temp_dir_rm(name, pid, instance);
 		return EXIT_FAILURE;
@@ -163,7 +166,7 @@ int stress_lease(
 	do {
 		fd = open(filename, O_WRONLY | O_APPEND, S_IRUSR | S_IWUSR);
 		if (fd < 0) {
-			pr_err(stderr, "%s: open failed: errno=%d: (%s)\n",
+			pr_err(stderr, "%s: open failed (parent): errno=%d: (%s)\n",
 				name, errno, strerror(errno));
 			goto reap;
 		}
@@ -178,7 +181,7 @@ int stress_lease(
 		(void)sched_yield();
 #endif
 		if (fcntl(fd, F_SETLEASE, F_UNLCK) < 0) {
-			pr_err(stderr, "%s: open failed: errno=%d: (%s)\n",
+			pr_err(stderr, "%s: fcntl failed: errno=%d: (%s)\n",
 				name, errno, strerror(errno));
 			(void)close(fd);
 			break;
