@@ -85,7 +85,7 @@ void stress_set_malloc_threshold(const char *optarg)
  */
 static inline size_t stress_alloc_size(void)
 {
-	size_t len = mwc() % opt_malloc_bytes;
+	size_t len = mwc64() % opt_malloc_bytes;
 
 	return len ? len : 1;
 }
@@ -126,6 +126,8 @@ int stress_malloc(
 again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -140,8 +142,8 @@ again:
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
-			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
-				name, WTERMSIG(status), instance);
+			pr_dbg(stderr, "%s: child died: %s (instance %d)\n",
+				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
@@ -161,7 +163,7 @@ again:
 		set_oom_adjustment(name, true);
 
 		do {
-			unsigned int rnd = mwc();
+			unsigned int rnd = mwc32();
 			unsigned int i = rnd % opt_malloc_max;
 			unsigned int action = (rnd >> 12) & 1;
 			unsigned int do_calloc = (rnd >> 14) & 0x1f;
