@@ -31,16 +31,16 @@
 #include <sys/time.h>
 #include "stress-ng.h"
 
-#define MATRIX_TYPE	float
+typedef float	matrix_type_t;
 
 /*
  *  the CPU stress test has different classes of cpu stressor
  */
 typedef void (*stress_matrix_func)(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n]);
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n]);
 
 typedef struct {
 	const char		*name;	/* human readable form of stressor */
@@ -67,11 +67,11 @@ void stress_set_matrix_size(const char *optarg)
  *  stress_matrix_prod(void)
  *	matrix product
  */
-static void stress_matrix_prod(
+static void OPTIMIZE3 stress_matrix_prod(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	size_t i;
 
@@ -94,11 +94,11 @@ static void stress_matrix_prod(
  *  stress_matrix_add(void)
  *	matrix addition
  */
-static void stress_matrix_add(
+static void OPTIMIZE3 stress_matrix_add(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
@@ -117,11 +117,11 @@ static void stress_matrix_add(
  *  stress_matrix_sub(void)
  *	matrix subtraction
  */
-static void stress_matrix_sub(
+static void OPTIMIZE3 stress_matrix_sub(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
@@ -140,11 +140,11 @@ static void stress_matrix_sub(
  *  stress_matrix_trans(void)
  *	matrix transpose
  */
-static void stress_matrix_trans(
+static void OPTIMIZE3 stress_matrix_trans(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],	/* Ignored */
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],	/* Ignored */
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
@@ -165,16 +165,16 @@ static void stress_matrix_trans(
  *  stress_matrix_mult(void)
  *	matrix scalar multiply
  */
-static void stress_matrix_mult(
+static void OPTIMIZE3 stress_matrix_mult(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
 	(void)b;
-	MATRIX_TYPE v = b[0][0];
+	matrix_type_t v = b[0][0];
 
 	for (i = 0; i < n; i++) {
 		register size_t j;
@@ -191,16 +191,16 @@ static void stress_matrix_mult(
  *  stress_matrix_div(void)
  *	matrix scalar divide
  */
-static void stress_matrix_div(
+static void OPTIMIZE3 stress_matrix_div(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
 	(void)b;
-	MATRIX_TYPE v = b[0][0];
+	matrix_type_t v = b[0][0];
 
 	for (i = 0; i < n; i++) {
 		register size_t j;
@@ -218,11 +218,11 @@ static void stress_matrix_div(
  *	matrix hadamard product
  *	(A o B)ij = AijBij
  */
-static void stress_matrix_hadamard(
+static void OPTIMIZE3 stress_matrix_hadamard(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
 
@@ -242,14 +242,14 @@ static void stress_matrix_hadamard(
  *	matrix frobenius product
  *	A : B = Sum(AijBij)
  */
-static void stress_matrix_frobenius(
+static void OPTIMIZE3 stress_matrix_frobenius(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	register size_t i;
-	MATRIX_TYPE sum = 0.0;
+	matrix_type_t sum = 0.0;
 
 	(void)r;
 
@@ -269,11 +269,11 @@ static void stress_matrix_frobenius(
  *  stress_matrix_all()
  *	iterate over all cpu stressors
  */
-static void stress_matrix_all(
+static void OPTIMIZE3 stress_matrix_all(
 	const size_t n,
-	MATRIX_TYPE a[n][n],
-	MATRIX_TYPE b[n][n],
-	MATRIX_TYPE r[n][n])
+	matrix_type_t a[n][n],
+	matrix_type_t b[n][n],
+	matrix_type_t r[n][n])
 {
 	static int i = 1;	/* Skip over stress_matrix_all */
 
@@ -335,7 +335,7 @@ int stress_matrix(
 {
 	stress_matrix_func func = opt_matrix_stressor->func;
 	size_t n;
-	const MATRIX_TYPE v = 1 / (MATRIX_TYPE)((uint32_t)~0);
+	const matrix_type_t v = 1 / (matrix_type_t)((uint32_t)~0);
 
 	(void)instance;
 	(void)name;
@@ -350,7 +350,7 @@ int stress_matrix(
 
 	{
 		register size_t i;
-		MATRIX_TYPE a[n][n], b[n][n], r[n][n];
+		matrix_type_t a[n][n], b[n][n], r[n][n];
 		/*
 		 *  Initialise matrices
 		 */
@@ -358,8 +358,8 @@ int stress_matrix(
 			register size_t j;
 
 			for (j = 0; j < n; j++) {
-				a[i][j] = (MATRIX_TYPE)mwc() * v;
-				b[i][j] = (MATRIX_TYPE)mwc() * v;
+				a[i][j] = (matrix_type_t)mwc64() * v;
+				b[i][j] = (matrix_type_t)mwc64() * v;
 				r[i][j] = 0.0;
 			}
 		}
