@@ -124,13 +124,18 @@ int stress_mq(
 		return EXIT_FAILURE;
 	}
 	if (sz < opt_mq_size) {
-		pr_inf(stdout, "%s: POSIX message queue requested size %d messages, maximum of %d allowed\n", name, opt_mq_size, sz);
+		pr_inf(stdout, "%s: POSIX message queue requested "
+			"size %d messages, maximum of %d allowed\n",
+			name, opt_mq_size, sz);
 	}
 	pr_dbg(stderr, "POSIX message queue %s with %lu messages\n",
 		mq_name, (unsigned long)attr.mq_maxmsg);
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -159,6 +164,7 @@ int stress_mq(
 		}
 	} else {
 		int status;
+		int attr_count = 0;
 		msg_t msg;
 
 		/* Parent */
@@ -166,10 +172,16 @@ int stress_mq(
 			memset(&msg, 0, sizeof(msg));
 			msg.value = (*counter);
 			msg.stop = false;
+			if ((attr_count++ & 31) == 0) {
+				struct mq_attr attr;
+
+				if (mq_getattr(mq, &attr) < 0)
+					pr_failed_dbg(name, "mq_getattr");
+			}
 
 			if (mq_send(mq, (char *)&msg, sizeof(msg), 1) < 0) {
 				if (errno != EINTR)
-					pr_failed_dbg(name, "msgsnd");
+					pr_failed_dbg(name, "mq_send");
 				break;
 			}
 			(*counter)++;
@@ -179,7 +191,7 @@ int stress_mq(
 		msg.stop = true;
 
 		if (mq_send(mq, (char *)&msg, sizeof(msg), 1) < 0) {
-			pr_failed_dbg(name, "termination msgsnd");
+			pr_failed_dbg(name, "termination mq_send");
 		}
 		(void)kill(pid, SIGKILL);
 		(void)waitpid(pid, &status, 0);
