@@ -81,8 +81,11 @@ int stress_pipe(
 		return EXIT_FAILURE;
 	}
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
 		pr_failed_dbg(name, "fork");
@@ -139,10 +142,13 @@ int stress_pipe(
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
 
 		strncpy(buf, PIPE_STOP, sizeof(buf));
-		if (write(pipefds[1], buf, sizeof(buf)) <= 0)
-			pr_failed_dbg(name, "termination write");
+		if (write(pipefds[1], buf, sizeof(buf)) <= 0) {
+			if (errno != EPIPE)
+				pr_failed_dbg(name, "termination write");
+		}
 		(void)kill(pid, SIGKILL);
 		(void)waitpid(pid, &status, 0);
+		(void)close(pipefds[1]);
 	}
 	return EXIT_SUCCESS;
 }
