@@ -101,8 +101,11 @@ int stress_poll(
 		}
 	}
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		ret = EXIT_FAILURE;
 		goto tidy;
@@ -117,7 +120,7 @@ int stress_poll(
 			ssize_t ret;
 
 			/* Write on a randomly chosen pipe */
-			i = (mwc() >> 8) % MAX_PIPES;
+			i = (mwc32() >> 8) % MAX_PIPES;
 			memset(buf, '0' + i, sizeof(buf));
 			ret = write(pipefds[i][1], buf, sizeof(buf));
 			if (ret < (ssize_t)sizeof(buf)) {
