@@ -34,6 +34,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/ipc.h>
@@ -66,24 +67,24 @@ void stress_semaphore_sysv_init(void)
 	int count = 0;
 
 	while (count < 100) {
-		shared->sem_sysv_key_id = (key_t)(mwc() & 0xffff);
-		shared->sem_sysv_id = semget(shared->sem_sysv_key_id, 1, IPC_CREAT | S_IRUSR | S_IWUSR);
-		if (shared->sem_sysv_id >= 0)
+		shared->sem_sysv.key_id = (key_t)mwc16();
+		shared->sem_sysv.sem_id = semget(shared->sem_sysv.key_id, 1, IPC_CREAT | S_IRUSR | S_IWUSR);
+		if (shared->sem_sysv.sem_id >= 0)
 			break;
 
 		count++;
 	}
 
-	if (shared->sem_sysv_id >= 0) {
+	if (shared->sem_sysv.sem_id >= 0) {
 		semun_t arg;
 
 		arg.val = 1;
-		if (semctl(shared->sem_sysv_id, 0, SETVAL, arg) == 0) {
-			shared->sem_sysv_init = true;
+		if (semctl(shared->sem_sysv.sem_id, 0, SETVAL, arg) == 0) {
+			shared->sem_sysv.init = true;
 			return;
 		}
 		/* Clean up */
-		(void)semctl(shared->sem_sysv_id, 0, IPC_RMID);
+		(void)semctl(shared->sem_sysv.sem_id, 0, IPC_RMID);
 	}
 
 	if (opt_sequential) {
@@ -103,8 +104,8 @@ void stress_semaphore_sysv_init(void)
  */
 void stress_semaphore_sysv_destroy(void)
 {
-	if (shared->sem_sysv_init)
-		(void)semctl(shared->sem_sysv_id, 0, IPC_RMID);
+	if (shared->sem_sysv.init)
+		(void)semctl(shared->sem_sysv.sem_id, 0, IPC_RMID);
 }
 
 /*
@@ -118,6 +119,13 @@ static void semaphore_sysv_thrash(
 {
 	do {
 		int i;
+		struct timespec timeout;
+
+		if (clock_gettime(CLOCK_REALTIME, &timeout) < 0) {
+			pr_failed_dbg(name, "clock_gettime");
+			return;
+		}
+		timeout.tv_sec++;
 
 		for (i = 0; i < 1000; i++) {
 			struct sembuf semwait, semsignal;
@@ -130,17 +138,23 @@ static void semaphore_sysv_thrash(
 			semsignal.sem_op = 1;
 			semsignal.sem_flg = SEM_UNDO;
 
-			if (semop(shared->sem_sysv_id, &semwait, 1) < 0) {
+			if (semtimedop(shared->sem_sysv.sem_id, &semwait, 1, &timeout) < 0) {
+				if (errno == EAGAIN) {
+					pr_inf(stderr, "Semaphore timed out: errno=%d (%s)\n",
+						errno, strerror(errno));
+					goto timed_out;
+				}
 				if (errno != EINTR)
 					pr_failed_dbg(name, "semop wait");
 				break;
 			}
 			(*counter)++;
-			if (semop(shared->sem_sysv_id, &semsignal, 1) < 0) {
+			if (semop(shared->sem_sysv.sem_id, &semsignal, 1) < 0) {
 				if (errno != EINTR)
 					pr_failed_dbg(name, "semop signal");
 				break;
 			}
+timed_out:
 			if (!opt_do_run)
 				break;
 		}
@@ -151,15 +165,18 @@ static void semaphore_sysv_thrash(
  *  semaphore_sysv_spawn()
  *	spawn a process
  */
-static int semaphore_sysv_spawn(
+static pid_t semaphore_sysv_spawn(
 	const char *name,
 	const uint64_t max_ops,
 	uint64_t *counter)
 {
 	pid_t pid;
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		return -1;
 	}
 	if (pid == 0) {
@@ -191,7 +208,7 @@ int stress_sem_sysv(
 			opt_semaphore_sysv_procs = MIN_SEMAPHORE_PROCS;
 	}
 
-	if (!shared->sem_sysv_init) {
+	if (!shared->sem_sysv.init) {
 		pr_err(stderr, "%s: aborting, semaphore not initialised\n", name);
 		return EXIT_FAILURE;
 	}
@@ -202,7 +219,7 @@ int stress_sem_sysv(
 		if (pids[i] < 0)
 			goto reap;
 	}
-	semaphore_sysv_thrash(name, max_ops, counter);
+	pause();
 reap:
 	for (i = 0; i < opt_semaphore_sysv_procs; i++) {
 		if (pids[i] > 0) {
