@@ -31,11 +31,14 @@
 #include <errno.h>
 #include <semaphore.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
 #include "stress-ng.h"
 
+#if defined(STRESS_SEMAPHORE_POSIX)
+
 static uint64_t opt_semaphore_posix_procs = DEFAULT_SEMAPHORE_PROCS;
 static bool set_semaphore_posix_procs = false;
 
@@ -54,8 +57,8 @@ void stress_set_semaphore_posix_procs(const char *optarg)
 void stress_semaphore_posix_init(void)
 {
 	/* create a mutex */
-	if (sem_init(&shared->sem_posix, 1, 1) >= 0) {
-		shared->sem_posix_init = true;
+	if (sem_init(&shared->sem_posix.sem, 1, 1) >= 0) {
+		shared->sem_posix.init = true;
 		return;
 	}
 
@@ -75,8 +78,8 @@ void stress_semaphore_posix_init(void)
  */
 void stress_semaphore_posix_destroy(void)
 {
-        if (shared->sem_posix_init) {
-		if (sem_destroy(&shared->sem_posix) < 0) {
+        if (shared->sem_posix.init) {
+		if (sem_destroy(&shared->sem_posix.sem) < 0) {
 			pr_err(stderr, "Semaphore destroy failed: errno=%d (%s)\n",
 				errno, strerror(errno));
 		}
@@ -94,14 +97,33 @@ static void semaphore_posix_thrash(
 {
 	do {
 		int i;
+		struct timespec timeout;
+
+		if (clock_gettime(CLOCK_REALTIME, &timeout) < 0) {
+			pr_failed_dbg(name, "clock_gettime");
+			return;
+		}
+		timeout.tv_sec++;
 
 		for (i = 0; i < 1000; i++) {
-			if (sem_wait(&shared->sem_posix) < 0) {
-				pr_failed_dbg(name, "sem_wait");
+			if (sem_timedwait(&shared->sem_posix.sem, &timeout) < 0) {
+				if (errno == ETIMEDOUT) {
+					/*
+					pr_inf(stderr, "Semaphore timed out: errno=%d (%s)\n",
+						errno, strerror(errno));
+					*/
+					goto timed_out;
+				}
+				if (errno != EINTR)
+					pr_failed_dbg(name, "sem_wait");
 				break;
 			}
 			(*counter)++;
-			sem_post(&shared->sem_posix);
+			if (sem_post(&shared->sem_posix.sem) < 0) {
+				pr_failed_dbg(name, "sem_post");
+				break;
+			}
+timed_out:
 			if (!opt_do_run)
 				break;
 		}
@@ -112,15 +134,18 @@ static void semaphore_posix_thrash(
  *  semaphore_posix_spawn()
  *	spawn a process
  */
-static int semaphore_posix_spawn(
+static pid_t semaphore_posix_spawn(
 	const char *name,
 	const uint64_t max_ops,
 	uint64_t *counter)
 {
 	pid_t pid;
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		return -1;
 	}
 	if (pid == 0) {
@@ -152,7 +177,7 @@ int stress_sem_posix(
 			opt_semaphore_posix_procs = MIN_SEMAPHORE_PROCS;
 	}
 
-	if (!shared->sem_posix_init) {
+	if (!shared->sem_posix.init) {
 		pr_err(stderr, "%s: aborting, semaphore not initialised\n", name);
 		return EXIT_FAILURE;
 	}
@@ -163,8 +188,8 @@ int stress_sem_posix(
 		if (pids[i] < 0)
 			goto reap;
 	}
-	semaphore_posix_thrash(name, max_ops, counter);
-
+	/* Wait for SIGALRM */
+	pause();
 reap:
 	for (i = 0; i < opt_semaphore_posix_procs; i++) {
 		if (pids[i] > 0) {
@@ -177,3 +202,5 @@ reap:
 
 	return EXIT_SUCCESS;
 }
+
+#endif
