@@ -54,19 +54,22 @@ int stress_sigfd(
 	(void)instance;
 
 	sigemptyset(&mask);
-	sigaddset(&mask, SIGUSR1);
+	sigaddset(&mask, SIGRTMIN);
 	if (sigprocmask(SIG_BLOCK, &mask, NULL) < 0) {
 		pr_failed_dbg(name, "sigprocmask");
-		_exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
 	sfd = signalfd(-1, &mask, 0);
 	if (sfd < 0) {
 		pr_failed_dbg(name, "signalfd");
-		_exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -76,7 +79,7 @@ int stress_sigfd(
 
 			memset(&s, 0, sizeof(s));
 			s.sival_int = 0;
-			ret = sigqueue(ppid, SIGUSR1, s);
+			ret = sigqueue(ppid, SIGRTMIN, s);
 			if (ret < 0)
 				break;
 		}
@@ -104,7 +107,7 @@ int stress_sigfd(
 			if (ret == 0)
 				break;
 			if (opt_flags & OPT_FLAGS_VERIFY) {
-				if (fdsi.ssi_signo != SIGUSR1) {
+				if (fdsi.ssi_signo != (uint32_t)SIGRTMIN) {
 					pr_fail(stderr, "%s: unexpected signal %d",
 						name, fdsi.ssi_signo);
 					break;
