@@ -36,7 +36,7 @@
 #include <signal.h>
 #include <sys/wait.h>
 
-static void stress_sigqhandler(int dummy)
+static void MLOCKED stress_sigqhandler(int dummy)
 {
 	(void)dummy;
 }
@@ -62,8 +62,11 @@ int stress_sigq(
 		return EXIT_FAILURE;
 	}
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
