@@ -63,14 +63,14 @@ void stress_set_socket_port(const char *optarg)
  */
 int stress_set_socket_domain(const char *name)
 {
-	return stress_set_net_domain("sock-domain", name, &opt_socket_domain);
+	return stress_set_net_domain(DOMAIN_ALL, "sock-domain", name, &opt_socket_domain);
 }
 
 /*
  *  handle_socket_sigalrm()
  *	catch SIGALRM
  */
-static void handle_socket_sigalrm(int dummy)
+static void MLOCKED handle_socket_sigalrm(int dummy)
 {
 	(void)dummy;
 	opt_do_run = false;
@@ -92,8 +92,11 @@ int stress_socket(
 	pr_dbg(stderr, "%s: process [%d] using socket port %d\n",
 		name, getpid(), opt_socket_port + instance);
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -139,6 +142,7 @@ retry:
 					break;
 				}
 			} while (opt_do_run && (!max_ops || *counter < max_ops));
+			(void)shutdown(fd, SHUT_RDWR);
 			(void)close(fd);
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
 
@@ -197,7 +201,22 @@ retry:
 			int sfd = accept(fd, (struct sockaddr *)NULL, NULL);
 			if (sfd >= 0) {
 				size_t i;
+				struct sockaddr addr;
+				socklen_t len;
+				int sndbuf;
 
+				len = sizeof(addr);
+				if (getsockname(fd, &addr, &len) < 0) {
+					pr_failed_dbg(name, "getsockname");
+					(void)close(sfd);
+					break;
+				}
+				len = sizeof(sndbuf);
+				if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, &len) < 0) {
+					pr_failed_dbg(name, "getsockopt");
+					(void)close(sfd);
+					break;
+				}
 				memset(buf, 'A' + (*counter % 26), sizeof(buf));
 				for (i = 16; i < sizeof(buf); i += 16) {
 					ssize_t ret = send(sfd, buf, i, 0);
@@ -207,6 +226,9 @@ retry:
 						break;
 					}
 				}
+				if (getpeername(sfd, &addr, &len) < 0) {
+					pr_failed_dbg(name, "getpeername");
+				}
 				(void)close(sfd);
 			}
 			(*counter)++;
