@@ -31,9 +31,6 @@
 #include <unistd.h>
 #include <signal.h>
 #include <errno.h>
-#if !defined(__FreeBSD__) && !defined(__OpenBSD__) && !defined(__NetBSD__)
-#include <alloca.h>
-#endif
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -45,7 +42,7 @@ static sigjmp_buf jmp_env;
  *  stress_segvhandler()
  *	SEGV handler
  */
-static void stress_segvhandler(int dummy)
+static void MLOCKED stress_segvhandler(int dummy)
 {
 	(void)dummy;
 
@@ -53,7 +50,6 @@ static void stress_segvhandler(int dummy)
 }
 
 
-
 /*
  *  stress_stack
  *	stress by forcing stack overflows
@@ -90,6 +86,8 @@ int stress_stack(
 again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -104,8 +102,8 @@ again:
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
-			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
-				name, WTERMSIG(status), instance);
+			pr_dbg(stderr, "%s: child died: %s (instance %d)\n",
+				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
