@@ -36,8 +36,10 @@
 
 static volatile uint64_t timer_counter = 0;
 static timer_t timerid;
-static uint64_t opt_timer_freq;
+static uint64_t opt_timer_freq = DEFAULT_TIMER_FREQ;
+static uint64_t overruns = 0;
 static bool set_timer_freq = false;
+static double rate_ns;
 
 /*
  *  stress_set_timer_freq()
@@ -51,26 +53,59 @@ void stress_set_timer_freq(const char *optarg)
 		MIN_TIMER_FREQ, MAX_TIMER_FREQ);
 }
 
+/*
+ *  stress_timer_set()
+ *	set timer, ensure it is never zero
+ */
+void stress_timer_set(struct itimerspec *timer)
+{
+	double rate;
+
+	if (opt_flags & OPT_FLAGS_TIMER_RAND) {
+		/* Mix in some random variation */
+		double r = ((double)(mwc32() % 10000) - 5000.0) / 40000.0;
+		rate = rate_ns + (rate_ns * r);
+	} else {
+		rate = rate_ns;
+	}
+
+	timer->it_value.tv_sec = (long long int)rate / 1000000000;
+	timer->it_value.tv_nsec = (long long int)rate % 1000000000;
+	if (timer->it_interval.tv_sec == 0 &&
+	    timer->it_interval.tv_nsec < 1)
+		timer->it_interval.tv_nsec = 1;
+
+	timer->it_interval.tv_sec = timer->it_value.tv_sec;
+	timer->it_interval.tv_nsec = timer->it_value.tv_nsec;
+}
+
 /*
  *  stress_timer_handler()
  *	catch timer signal and cancel if no more runs flagged
  */
-static void stress_timer_handler(int sig)
+static void MLOCKED stress_timer_handler(int sig)
 {
+	struct itimerspec timer;
+	int ret;
+
 	(void)sig;
 	timer_counter++;
 
-	/* Cancel timer if we detect no more runs */
-	if (!opt_do_run) {
-		struct itimerspec timer;
+	ret = timer_getoverrun(timerid);
+	if (ret > 0)
+		overruns += ret;
+
+	if (opt_do_run) {
+		stress_timer_set(&timer);
+	} else {
+		/* Cancel timer if we detect no more runs */
 
 		timer.it_value.tv_sec = 0;
 		timer.it_value.tv_nsec = 0;
-		timer.it_interval.tv_sec = timer.it_value.tv_sec;
-		timer.it_interval.tv_nsec = timer.it_value.tv_nsec;
-
-		timer_settime(timerid, 0, &timer, NULL);
+		timer.it_interval.tv_sec = 0;
+		timer.it_interval.tv_nsec = 0;
 	}
+	(void)timer_settime(timerid, 0, &timer, NULL);
 }
 
 /*
@@ -86,7 +121,6 @@ int stress_timer(
 	struct sigaction new_action;
 	struct sigevent sev;
 	struct itimerspec timer;
-	double rate_ns;
 
 	(void)instance;
 
@@ -114,11 +148,7 @@ int stress_timer(
 		return EXIT_FAILURE;
 	}
 
-	timer.it_value.tv_sec = (long long int)rate_ns / 1000000000;
-	timer.it_value.tv_nsec = (long long int)rate_ns % 1000000000;
-	timer.it_interval.tv_sec = timer.it_value.tv_sec;
-	timer.it_interval.tv_nsec = timer.it_value.tv_nsec;
-
+	stress_timer_set(&timer);
 	if (timer_settime(timerid, 0, &timer, NULL) < 0) {
 		pr_failed_err(name, "timer_settime");
 		return EXIT_FAILURE;
@@ -137,6 +167,8 @@ int stress_timer(
 		pr_failed_err(name, "timer_delete");
 		return EXIT_FAILURE;
 	}
+	pr_dbg(stderr, "%s: %" PRIu64 " timer overruns (instance %" PRIu32 ")\n",
+		name, overruns, instance);
 
 	return EXIT_SUCCESS;
 }
