@@ -63,14 +63,14 @@ void stress_set_udp_port(const char *optarg)
  */
 int stress_set_udp_domain(const char *name)
 {
-	return stress_set_net_domain("udp-domain", name, &opt_udp_domain);
+	return stress_set_net_domain(DOMAIN_ALL, "udp-domain", name, &opt_udp_domain);
 }
 
 /*
  *  handle_udp_sigalrm()
  *	catch SIGALRM
  */
-static void handle_udp_sigalrm(int dummy)
+static void MLOCKED handle_udp_sigalrm(int dummy)
 {
 	(void)dummy;
 	opt_do_run = false;
@@ -92,8 +92,11 @@ int stress_udp(
 	pr_dbg(stderr, "%s: process [%d] using udp port %d\n",
 		name, getpid(), opt_udp_port + instance);
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		pr_failed_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
