@@ -36,6 +36,8 @@
 #include "stress-ng.h"
 
 #define VM_BOGO_SHIFT		(12)
+#define VM_ROWHAMMER_LOOPS	(1000000)
+
 
 /*
  *  the VM stress test has diffent methods of vm stressor
@@ -215,17 +217,17 @@ static size_t stress_vm_moving_inversion(
 	buf_end = (uint64_t *)(buf + sz);
 
 	mwc_reseed();
-	w = mwc();
-	z = mwc();
+	w = mwc64();
+	z = mwc64();
 
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < buf_end; ) {
-		*(ptr++) = mwc();
+		*(ptr++) = mwc64();
 	}
 
 	mwc_seed(w, z);
 	for (bit_errors = 0, ptr = (uint64_t *)buf; ptr < buf_end; ) {
-		uint64_t val = mwc();
+		uint64_t val = mwc64();
 
 		if (*ptr != val)
 			bit_errors++;
@@ -243,7 +245,7 @@ static size_t stress_vm_moving_inversion(
 
 	mwc_seed(w, z);
 	for (bit_errors = 0, ptr = (uint64_t *)buf; ptr < buf_end; ) {
-		uint64_t val = mwc();
+		uint64_t val = mwc64();
 		if (*(ptr++) != ~val)
 			bit_errors++;
 		c++;
@@ -255,7 +257,7 @@ static size_t stress_vm_moving_inversion(
 
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
-		*--ptr = mwc();
+		*--ptr = mwc64();
 		c++;
 		if (max_ops && c >= max_ops)
 			goto abort;
@@ -266,7 +268,7 @@ static size_t stress_vm_moving_inversion(
 	(void)mincore_touch_pages(buf, sz);
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
-		uint64_t val = mwc();
+		uint64_t val = mwc64();
 		if (*--ptr != val)
 			bit_errors++;
 		*ptr = ~val;
@@ -278,7 +280,7 @@ static size_t stress_vm_moving_inversion(
 	}
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
-		uint64_t val = mwc();
+		uint64_t val = mwc64();
 		if (*--ptr != ~val)
 			bit_errors++;
 		if (!opt_do_run)
@@ -313,7 +315,7 @@ static size_t stress_vm_modulo_x(
 	uint64_t c = *counter;
 
 	mwc_reseed();
-	pattern = mwc();
+	pattern = mwc8();
 	compliment = ~pattern;
 
 	for (i = 0; i < stride; i++) {
@@ -704,8 +706,8 @@ static size_t stress_vm_swap(
 	size_t *swaps;
 
 	mwc_reseed();
-	z1 = mwc();
-	w1 = mwc();
+	z1 = mwc64();
+	w1 = mwc64();
 
 	if ((swaps = calloc(chunks, sizeof(size_t))) == NULL) {
 		pr_failed_dbg("stress-vm", "calloc");
@@ -713,12 +715,12 @@ static size_t stress_vm_swap(
 	}
 
 	for (i = 0; i < chunks; i++) {
-		swaps[i] = (mwc() % chunks) * chunk_sz;
+		swaps[i] = (mwc64() % chunks) * chunk_sz;
 	}
 
 	mwc_seed(w1, z1);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = mwc();
+		uint8_t val = mwc8();
 		memset((void *)ptr, val, chunk_sz);
 	}
 
@@ -767,7 +769,7 @@ static size_t stress_vm_swap(
 	mwc_seed(w1, z1);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
 		volatile uint8_t *p = ptr, *p_end = ptr + chunk_sz;
-		uint8_t val = mwc();
+		uint8_t val = mwc8();
 
 		while (p < p_end) {
 			if (*p != val)
@@ -803,12 +805,12 @@ static size_t stress_vm_rand_set(
 	size_t bit_errors = 0;
 
 	mwc_reseed();
-	w = mwc();
-	z = mwc();
+	w = mwc64();
+	z = mwc64();
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = mwc();
+		uint8_t val = mwc8();
 
 		*(ptr + 0) = val;
 		*(ptr + 1) = val;
@@ -830,7 +832,7 @@ static size_t stress_vm_rand_set(
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = mwc();
+		uint8_t val = mwc8();
 
 		bit_errors += (*(ptr + 0) != val);
 		bit_errors += (*(ptr + 1) != val);
@@ -870,12 +872,12 @@ static size_t stress_vm_ror(
 	size_t bit_errors = 0;
 
 	mwc_reseed();
-	w = mwc();
-	z = mwc();
+	w = mwc64();
+	z = mwc64();
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = (uint8_t)mwc();
+		uint8_t val = mwc8();
 
 		*(ptr + 0) = val;
 		*(ptr + 1) = val;
@@ -914,7 +916,7 @@ static size_t stress_vm_ror(
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = (uint8_t)mwc();
+		uint8_t val = mwc8();
 		ROR64(val);
 
 		bit_errors += (*(ptr + 0) != val);
@@ -955,12 +957,12 @@ static size_t stress_vm_flip(
 	size_t bit_errors = 0, i;
 
 	mwc_reseed();
-	w = mwc();
-	z = mwc();
+	w = mwc64();
+	z = mwc64();
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = (uint8_t)mwc();
+		uint8_t val = mwc8();
 
 		*(ptr + 0) = val;
 		ROR8(val);
@@ -1009,7 +1011,7 @@ static size_t stress_vm_flip(
 
 	mwc_seed(w, z);
 	for (ptr = buf; ptr < buf_end; ptr += chunk_sz) {
-		uint8_t val = (uint8_t)mwc();
+		uint8_t val = mwc8();
 
 		bit_errors += (*(ptr + 0) != val);
 		ROR8(val);
@@ -1123,8 +1125,8 @@ static size_t stress_vm_galpat_zero(
 
 	for (i = 0; i < bits_bad; i++) {
 		for (;;) {
-			size_t offset = mwc() % sz;
-			uint8_t bit = mwc() & 3;
+			size_t offset = mwc64() % sz;
+			uint8_t bit = mwc32() & 3;
 
 			if (!buf[offset]) {
 				buf[offset] |= (1 << bit);
@@ -1185,8 +1187,8 @@ static size_t stress_vm_galpat_one(
 
 	for (i = 0; i < bits_bad; i++) {
 		for (;;) {
-			size_t offset = mwc() % sz;
-			uint8_t bit = mwc() & 3;
+			size_t offset = mwc64() % sz;
+			uint8_t bit = mwc32() & 3;
 
 			if (buf[offset] == 0xff) {
 				buf[offset] &= ~(1 << bit);
@@ -1316,19 +1318,19 @@ static size_t stress_vm_rand_sum(
 	size_t bit_errors = 0;
 
 	mwc_reseed();
-	w = mwc();
-	z = mwc();
+	w = mwc64();
+	z = mwc64();
 
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < buf_end; ptr += chunk_sz) {
-		*(ptr + 0) = mwc();
-		*(ptr + 1) = mwc();
-		*(ptr + 2) = mwc();
-		*(ptr + 3) = mwc();
-		*(ptr + 4) = mwc();
-		*(ptr + 5) = mwc();
-		*(ptr + 6) = mwc();
-		*(ptr + 7) = mwc();
+		*(ptr + 0) = mwc64();
+		*(ptr + 1) = mwc64();
+		*(ptr + 2) = mwc64();
+		*(ptr + 3) = mwc64();
+		*(ptr + 4) = mwc64();
+		*(ptr + 5) = mwc64();
+		*(ptr + 6) = mwc64();
+		*(ptr + 7) = mwc64();
 		c++;
 		if (max_ops && c >= max_ops)
 			goto abort;
@@ -1341,14 +1343,14 @@ static size_t stress_vm_rand_sum(
 
 	mwc_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < buf_end; ptr += chunk_sz) {
-		bit_errors += stress_vm_count_bits(*(ptr + 0) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 1) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 2) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 3) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 4) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 5) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 6) ^ mwc());
-		bit_errors += stress_vm_count_bits(*(ptr + 7) ^ mwc());
+		bit_errors += stress_vm_count_bits(*(ptr + 0) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 1) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 2) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 3) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 4) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 5) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 6) ^ mwc64());
+		bit_errors += stress_vm_count_bits(*(ptr + 7) ^ mwc64());
 		if (!opt_do_run)
 			break;
 	}
@@ -1729,6 +1731,70 @@ static size_t stress_vm_read64(
 	return 0;
 }
 
+/*
+ *  stress_vm_rowhammer()
+ *
+ */
+static size_t stress_vm_rowhammer(
+	uint8_t *buf,
+	const size_t sz,
+	uint64_t *counter,
+	const uint64_t max_ops)
+{
+	size_t bit_errors = 0;
+	uint32_t *buf32 = (uint32_t *)buf;
+	uint32_t val = 0xff5a00a5;
+	const size_t n = sz / sizeof(uint32_t);
+	register size_t j;
+
+	(void)mincore_touch_pages(buf, sz);
+
+	do {
+		for (j = 0; j < n; j++)
+			buf32[j] = val;
+		register volatile uint32_t *addr0, *addr1;
+		register size_t errors = 0;
+
+		/* Pick two random addresses */
+		addr0 = &buf32[(mwc64() << 12) % n];
+		addr1 = &buf32[(mwc64() << 12) % n];
+
+		/* Hammer the rows */
+		for (j = VM_ROWHAMMER_LOOPS / 4; j; j--) {
+			*addr0;
+			*addr1;
+			clflush(addr0);
+			clflush(addr1);
+			*addr0;
+			*addr1;
+			clflush(addr0);
+			clflush(addr1);
+			*addr0;
+			*addr1;
+			clflush(addr0);
+			clflush(addr1);
+			*addr0;
+			*addr1;
+			clflush(addr0);
+			clflush(addr1);
+		}
+		for (j = 0; j < n; j++)
+			if (buf32[j] != val)
+				errors++;
+		if (errors) {
+			bit_errors += errors;
+			pr_dbg(stderr, "rowhammer: %zu errors on addresses %p and %p\n",
+				errors, addr0, addr1);
+		}
+		(*counter) += VM_ROWHAMMER_LOOPS;
+		val = (val >> 31) | (val << 1);
+	} while (opt_do_run && (!max_ops || *counter < max_ops));
+
+	stress_vm_check("rowhammer", bit_errors);
+
+	return bit_errors;
+}
+
 /*
  *  stress_vm_all()
  *	work through all vm stressors sequentially
@@ -1756,6 +1822,7 @@ static stress_vm_stressor_info_t vm_methods[] = {
 	{ "galpat-0",	stress_vm_galpat_zero },
 	{ "galpat-1",	stress_vm_galpat_one },
 	{ "gray",	stress_vm_gray },
+	{ "rowhammer",	stress_vm_rowhammer },
 	{ "incdec",	stress_vm_incdec },
 	{ "inc-nybble",	stress_vm_inc_nybble },
 	{ "rand-set",	stress_vm_rand_set },
@@ -1837,6 +1904,8 @@ again:
 		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
+		if (errno == EAGAIN)
+			goto again;
 		pr_err(stderr, "%s: fork failed: errno=%d: (%s)\n",
 			name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -1850,8 +1919,8 @@ again:
 			(void)kill(pid, SIGKILL);
 			(void)waitpid(pid, &status, 0);
 		} else if (WIFSIGNALED(status)) {
-			pr_dbg(stderr, "%s: child died: %d (instance %d)\n",
-				name, WTERMSIG(status), instance);
+			pr_dbg(stderr, "%s: child died: %s (instance %d)\n",
+				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
@@ -1866,7 +1935,7 @@ again:
 		set_oom_adjustment(name, true);
 
 		do {
-			if (!keep || (keep && buf == NULL)) {
+			if (!keep || (buf == NULL)) {
 				if (!opt_do_run)
 					return EXIT_SUCCESS;
 				buf = mmap(NULL, buf_sz, PROT_READ | PROT_WRITE,
