@@ -38,23 +38,27 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#define ABORT_TIMEOUT	(2.0)
+#define ABORT_TIMEOUT	(8.0)
 
 /*
  *  spawn()
  *	spawn a process
  */
-static int spawn(
+static pid_t spawn(
 	const char *name,
-	void (*func)(const char *name, const pid_t pid, uint64_t *counter, const uint64_t max_ops),
+	void (*func)(const char *name, const pid_t pid,
+		uint64_t *counter, const uint64_t max_ops),
 	pid_t pid_arg,
 	uint64_t *counter,
 	uint64_t max_ops)
 {
 	pid_t pid;
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (opt_do_run && (errno == EAGAIN))
+			goto again;
 		return -1;
 	}
 	if (pid == 0) {
@@ -69,7 +73,11 @@ static int spawn(
  *	this process pauses, but is continually being
  *	stopped and continued by the killer process
  */
-static void runner(const char *name, const pid_t pid, uint64_t *counter, const uint64_t max_ops)
+static void runner(
+	const char *name,
+	const pid_t pid,
+	uint64_t *counter,
+	const uint64_t max_ops)
 {
 	(void)pid;
 
@@ -87,7 +95,11 @@ static void runner(const char *name, const pid_t pid, uint64_t *counter, const u
  *  killer()
  *	this continually stops and continues the runner process
  */
-static void killer(const char *name, const pid_t pid, uint64_t *counter, const uint64_t max_ops)
+static void killer(
+	const char *name,
+	const pid_t pid,
+	uint64_t *counter,
+	const uint64_t max_ops)
 {
 	double start = time_now();
 	uint64_t last_counter = *counter;
