@@ -24,14 +24,15 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_YIELD)
+
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-
-#if defined(_POSIX_PRIORITY_SCHEDULING)
 #include <sched.h>
-#include "stress-ng.h"
 
 /*
  *  stress on sched_yield()
@@ -51,8 +52,8 @@ int stress_yield(
 
 		ret = sched_yield();
 		if ((ret < 0) && (opt_flags & OPT_FLAGS_VERIFY))
-			pr_fail(stderr, "sched_yield failed: errno=%d (%s)\n",
-				errno, strerror(errno));
+			pr_fail(stderr, "%s: sched_yield failed: errno=%d (%s)\n",
+				name, errno, strerror(errno));
 		(*counter)++;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
 
